package org.openxma.addons.ui.table.customizer.xma.client;

import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;

import at.spardat.xma.mdl.Atom;

/**
 * The context to the given contextmenu, determined by the referenced column and item.
 * Contains convenience methods retrieve the column, the item, the itemkey as well as the cell and cellvalue.
 * 
 * @see #getTableColumn()
 * @see #getTableItem()
 * @see #getItemKey()
 * @see #getCell()
 * @see #getCellValue()
 * 
 * @since 5.0.6
 */
public class Context {

    private final TableCustomizerComp customizer;
    
    private TableColumn tableColumn;
    private TableItem tableItem;
    private Atom cell;
    
    int idxItemUI;
    int idxItemModel;
    int idxColUI;
    int idxColModel;
    
    Context(TableCustomizerComp customizer) {
        this.customizer = customizer;
    }
    
    /**
     * Key function that updates the context.
     * Needs to be called at least once.
     * 
     * @param column
     * @param item
     */
    void update(TableColumn tableColumn, TableItem tableItem) {
        this.tableColumn = tableColumn;
        updateColumn();
        
        this.tableItem = tableItem;
        updateItem();
        
        cell = null; //will recalculate automatically on next get
    }
    
    private void updateColumn() {
        if (tableColumn != null) {
            idxColUI = customizer.getTable().indexOf(tableColumn);
            idxColModel = customizer.getTableWMClient().uiColumnIndexToModelColumnIndex(idxColUI);
        } else {
            idxColUI = -1;
            idxColModel = -1;
        }
    }
    
    private void updateItem() {
        if (tableItem != null) {
            idxItemUI = customizer.getTable().indexOf(tableItem);
            idxItemModel = customizer.getTableWMClient().uiIndex2ModelIndex(idxItemUI);
        } else {
            idxItemUI = -1;
            idxItemModel = -1;
        }
        
    }

    /**
     * @return The TableColumn referenced by the contextmenu. May be null.
     */
    public TableColumn getTableColumn() {
        return tableColumn;
    }

    /**
     * @return The TableItem referenced by the contextmenu. May be null.
     */
    public TableItem getTableItem() {
        return tableItem;
    }

    /**
     * @return The key of the item referenced by the contextmenu. Null if there is no item for this context.
     */
    public String getItemKey() {
        return idxItemModel != -1 ? customizer.getTableWMClient().getRow(idxItemModel).getKey() : null;
    }
    

    /**
     * 
     * @return The cell placed in the given column and item. May be null.
     * 
     * @see #getTableColumn()
     * @see #getTableItem()
     */
    public Atom getCell() {
        if (idxItemModel != -1 && cell == null) {
            cell = customizer.getTableWMClient().getRow(idxItemModel).getCell(idxColModel);
        }
        return cell;   
    }

    /**
     * @return The external value (as displayed) from the given cell. Null if no there is no specific cell for this context.
     * @see #getCell
     */
    public String getExternalValue() {
        return getCell() != null ? customizer.getTableWMClient().getFormatter(idxColModel).format(getCell().getEncodedValue()) : null;
    }
    
    /**
     * @return The internal value (as saved by the DB) from the given cell. Null if no there is no specific cell for this context.
     * @see #getCell
     */
    public String getInternalValue() {
        return getCell() != null ? getCell().getEncodedValue() : null;
    }
}